<?php
header('Content-type: text/plain', true);

include_once '../includes/aibridge.class.php';
include_once '../includes/globals.php';
include_once '../includes/rosetta.class.php';
include_once '../includes/filesaver.class.php';

// Check for required parameters
if (!requiredParameters('target', 'mount', 'device', 'filename'))
{
  $outputArray = array(
    'name'  => 'Error',
    'value' => 'Some required parameters are missing');
  die(Rosetta::arrayToJSON($outputArray));
}

// See if file exists
if ((!isset($_GET['overwrite']) || $_GET['overwrite'] != 'yes') &&
    FileSaver::doesFileExist($_GET['filename'], 'html', $_GET['target'], $_GET['mount'], $_GET['device']))
{
  $outputArray = array(
    'name'  => 'Error',
    'value' => 'error:exists');
  die(Rosetta::arrayToJSON($outputArray));
}

// Call AI
$stdout = AIBridge::getDiagnosisLog();

// Try to make stdout into a DomDocument
$xml = new DomDocument($stdout);
if ($xml)
{
  $document = $xml->document_element();
  if ($document->tagname() == 'error') // If AI returned an error:
  {
    $outputArray = array(
      'name'  => 'AI Error',
      'value' => $document->get_content());
    die(Rosetta::arrayToJSON($outputArray));
  }
  $xpath = xpath_new_context($document);
$records = @ xpath_eval($xpath, '/diagnosisLogRecord');
}
else
{
  $outputArray = array(
    'name'  => 'XML Error',
    'value' => $stdout);
  die(Rosetta::arrayToJSON($outputArray));
}

// Output HTML
$computerName = getComputerName();
$buildVersion = getBuildVersion();
$html = array();

$html[] = '<html>';
$html[] = '<head>';
$html[] = '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">';
$html[] = '<title>' . $computerName . ' - ' . translate('MSG_DIAGNOSIS_LOG') . '</title>';
$html[] = '<style type="text/css">';
$html[] = 'body';
$html[] = '{';
  $html[] = 'padding:10px;';
  $html[] = 'margin: 0;';
  $html[] = 'font-size:12px;';
  $html[] = 'font-family:Helvetica,Arial,sans-serif;';
$html[] = '}';
$html[] = 'table';
$html[] = '{';
  $html[] = 'margin:0;';
  $html[] = 'padding:0;'; 
  $html[] = 'border-collapse:collapse;';
  $html[] = 'border:3px solid #968F89;';
  $html[] = 'font-size:1em;';
  $html[] = 'width:100%;';
$html[] = '}';
$html[] = 'tr.m td';
$html[] = '{';
  $html[] = 'background-color:#EEE;';
  $html[] = 'text-align:center;';
  $html[] = 'font-size:1.2em;';
  $html[] = 'font-weight:bold;';
$html[] = '}';
$html[] = 'thead td';
$html[] = '{';
  $html[] = 'border:none;';
  $html[] = 'background-color:#968F89;';
  $html[] = 'white-space:nowrap;';
  $html[] = 'color:#FFFFFF;';
  $html[] = 'font-weight:bold;';
  $html[] = 'padding-right:5px;';
$html[] = '}';
$html[] = 'td,th';
$html[] = '{';
  $html[] = 'border-right:1px solid #EEE;';
  $html[] = 'border-bottom:1px solid #999;';
  $html[] = 'text-align:left;';
  $html[] = 'padding-left:2px;';
  $html[] = 'vertical-align:top;';
$html[] = '}';
$html[] = 'td.c';
$html[] = '{';
  $html[] = 'text-align:center;';
$html[] = '}';
$html[] = 'td.r';
$html[] = '{';
  $html[] = 'text-align:right;';
$html[] = '}';
$html[] = '.p';
$html[] = '{';
  $html[] = 'width:15px;';
  $html[] = 'background-color:#0F0;';
$html[] = '}';
$html[] = '.f';
$html[] = '{';
  $html[] = 'width:15px;';
  $html[] = 'background-color:#F00;';
$html[] = '}';
$html[] = '.p span,.f span';
$html[] = '{';
  $html[] = 'display:none;';
$html[] = '}';
$html[] = 'table.i';
$html[] = '{';
  $html[] = 'border:none;';
  $html[] = 'margin:3px 0;';
$html[] = '}';
$html[] = 'table.i td';
$html[] = '{';
  $html[] = 'border:none;';
$html[] = '}';
$html[] = '</style>';
$html[] = '</head>';
$html[] = '<body>';

if ($buildVersion != '')
  $html[] = ('<i>HP Insight Diagnostics ' . translate('MSG_VERSION') . ' ' . $buildVersion . '</i><br>');

if ($computerName != '')
  $html[] = (translate('MSG_COMPUTER_NAME') . ': <b>' . $computerName . '</b>');

if (count($records->nodeset) > 0)
{
  $html[] = '<table class="i">';
  $html[] = '<tr>';
  $html[] = ('<td class="p"><span>' . translate('MSG_PASSED') . '</span></td>');
  $html[] = ('<td> - ' . translate('MSG_DENOTES_PASSED_TESTS') . '</td>');
  $html[] = '</tr>';
  $html[] = '<tr>';
  $html[] = ('<td class="f"><span>' . translate('MSG_FAILED') . '</span></td>');
  $html[] = ('<td> - ' . translate('MSG_DENOTES_FAILED_TESTS') . '</td>');
  $html[] = '</tr>';
  $html[] = '</table>';
}
  
$html[] = '<table>';

if (count($records->nodeset) <= 0)
{
  $html[] = '<tr class="m">';
  $html[] = ('<td>' . translate('MSG_DIAGNOSIS_LOG') . ' ' . translate('MSG_IS_EMPTY') . '</td>');
  $html[] = '<tr>';
}
else
{
  $html[] = '<thead>';
  $html[] = '<tr>';
  $html[] = '<td>&nbsp;</td>';
  $html[] = ('<td>' . translate('MSG_DEVICE') . ', ' . translate('MSG_TEST') . '</td>');
  $html[] = ('<td class="c">' . translate('MSG_NUMBER_OF_TIMES_DIAGNOSED') . '</td>');
  $html[] = ('<td class="c">' . translate('MSG_FAILED') . ' ' . translate('MSG_COUNT') . '</td>');
  $html[] = ('<td class="r">' . translate('MSG_DIAGNOSIS_TIME') . '</td>');
  $html[] = '</tr>';
  $html[] = '</thead>';
  $html[] = '<tbody>';

  foreach ($records->nodeset as $record)
  {
    $test = $record->get_attribute('diagnosisCaption');
    $passedCount = $record->get_attribute('passedCount');
    $failedCount = $record->get_attribute('failedCount');
    $numberOfTests = ($passedCount + $failedCount);
    $time = $record->get_attribute('testTime');
    $result = ($failedCount > 0) ? 'failed' : 'passed';
    $resultCaption = ($result == 'passed') ? translate('MSG_PASSED') : translate('MSG_FAILED');
    $resultClass = ($result == 'passed') ? ' class="p"' : ' class="f"';

    $html[] = ('<tr>');
    $html[] = ('<td' . $resultClass . '><span>' . $resultCaption . '</span></td>');
    $html[] = ('<td>' . $test . '</td>');
    $html[] = ('<td class="c">' . $numberOfTests . '</td>');
    $html[] = ('<td class="c">' . $failedCount . '</td>');
    $html[] = ('<td class="r">' . $time . '</td>');
    $html[] = '</tr>';
  }
  $html[] = '</tbody>';
}

$html[] = '</table>';
$html[] = '</body>';
$html[] = '</html>';

$saveResult = FileSaver::saveArrayToFile($_GET['filename'], 'html', $_GET['target'], $_GET['mount'], $_GET['device'], $html);
if ($saveResult == 'success')
{
  exit('0');
}
else
{
  $outputArray = array(
    'name'  => 'Error',
    'value' => $saveResult);
  die(Rosetta::arrayToJSON($outputArray));
}
//*/
?>
