<?php
  $systemConstants = get_defined_constants(); // used in debug mode as a filter

  require_once 'constants.php';
  require_once 'translations.php';

  /* DECLARE ADDITIONAL CONSTANTS */

  if (checkDiagConfig('/factory'))
    define('IS_FACTORY', true);
  else
    define('IS_FACTORY', false);

  if (isset($_REQUEST['man']) && $_REQUEST['man'] == '1')
    define('MANUAL_OVERRIDE', true);
  else
    define('MANUAL_OVERRIDE', false);

  if (isSurveyOnly())
    define('SURVEY_ONLY', true);
  else
    define('SURVEY_ONLY', false);

  if (isImlSupported())
    define('IML_SUPPORT', true);
  else
    define('IML_SUPPORT', false);

  if (hasDiagnosableDevices())
    define('DIAGNOSE_SUPPORT', true);
  else
    define('DIAGNOSE_SUPPORT', false);

  /* GLOBAL FUNCTIONS */

  function frontendStartup()
  {
    $path = $_SERVER['SCRIPT_URI'];
    $lastSlash = strrpos($path, '/');
    $baseURI = substr($path, 0, ($lastSlash + 1));
    define('FRONTEND_DIRECTORY', $baseURI);

    $idResult = new DomDocument(file_get_contents(IDRESULT_PATH));
    $xpath = xpath_new_context($idResult->document_element());
    if (IS_WINDOWS)
      $x64 = @ xpath_eval($xpath, '/diagOutput/category[@name="operatingSystem"]/structure[@name="osVenderVersion"]/property[@name="osSystemInfo"]');
    else
      $x64 = @ xpath_eval($xpath, '/diagOutput/category[@name="operatingSystem"]/structure[@name="operatingSystem"]/property[@name="osCommercialName"]');

    if (count($x64->nodeset) > 0 && strpos($x64->nodeset[0]->get_attribute('value'), '64') !== false)
      define('IS_X64', true);
    else
      define('IS_X64', false);

    $diagOutput = @ xpath_eval($xpath, '/diagOutput');
    if (count($diagOutput->nodeset) > 0)
    {
      $diagOutput = $diagOutput->nodeset[0];
      define('DIAGS_VERSION', $diagOutput->get_attribute('buildVersion'));
      define('DIAGS_BUILD_DATE', $diagOutput->get_attribute('buildDate'));
    }
    else
    {
      define('DIAGS_VERSION', '');
      define('DIAGS_BUILD_DATE', '');
    }
    
    define('DIAGS_COPYRIGHT', (translate('MSG_COPYRIGHT') . ' &copy; 2002 - 2007 Hewlett-Packard Development Company, L.P.'));

    $runMeNowTests = @ xpath_eval($xpath, '/diagOutput/category/device/test[@runMeNow="1"]');
    $numberOfTests = count($runMeNowTests->nodeset);
    if ($numberOfTests > 0)
    {
      $testsArray = array();
      foreach ($runMeNowTests->nodeset as $test)
      {
        $testArray = array();
        $parent = $test->parent_node();
        $testArray['test'] = $test->get_attribute('name');
        $testArray['device'] = $parent->get_attribute('name');
        $testsArray[] = $testArray;
      }
      require_once 'aibridge.class.php';
      $stdout = AIBridge::startCustomTests('', '', '', '', $testsArray, true);
      $xml = new DomDocument($stdout);
      if ($xml)
      {
        $root = $xml->document_element();
        if ($root->tagname() != 'error')
        {
          define('SCRIPT_TESTING', true);
          define('SCRIPT_TESTING_PID', $root->get_attribute('processID'));
        }
        else
        {
          define('SCRIPT_TESTING', false);
          define('SCRIPT_TESTING_ERROR', $root->get_content());
        }
      }
      else
      {
        define('SCRIPT_TESTING', false);
        define('SCRIPT_TESTING_ERROR', ('XMLDOC Error: ' . $stdout));
      }
    }

    if (!defined('SCRIPT_TESTING') &&      // Make sure we aren't running script testing already (runMeNow tests)
        MANUAL_OVERRIDE != true &&         // Make sure manual override has not been declared
        checkDiagConfig('/src') == true && // Make sure we have /src files defined in diagconfig.xml
        IS_FACTORY == true)                // Make sure we are in factory mode
    {
      $diagconfig = new DomDocument(file_get_contents(DIAGS_DIRECTORY . 'diagconfig.xml'));
      $xpath = xpath_new_context($diagconfig->document_element());
      $srcs = @ xpath_eval($xpath, '/src');
      $src = $srcs->nodeset[0];
      $srcFilename = $src->get_attribute('name');
      $srcDirectory = $src->get_attribute('path');
      if ($srcDirectory == '')
        $srcDirectory = DIAGS_DIRECTORY;

      if (strstr($srcFilename, 'factory.xml'))
      {
        // Create factory.xml if it is missing
        if (!file_exists($srcDirectory . 'factory.xml'))
        {
          chdir($srcDirectory);
          $files = glob('*.src.xml');
          $srcXML = $files[0];
          require_once 'aibridge.class.php';
          AIBridge::createFactoryXML($srcXML);
        }
        $factoryXML = new DomDocument(file_get_contents($srcDirectory . $srcFilename));
        $factoryRoot = $factoryXML->document_element();
        $xpath = xpath_new_context($factoryRoot);
        $station = $factoryRoot->get_attribute('station');
        $srcFileNodeset = @ xpath_eval($xpath, ('/src[@station="' . $station . '"]'));
        $srcFile = $srcFileNodeset->nodeset[0];
        $status = $srcFile->get_attribute('status');
        // if we've passed this station, move to the next one
        if (strtolower($status) == 'passed')
        {
          $station++;
          $srcFileNodeset = @ xpath_eval($xpath, ('/src[@station="' . $station . '"]'));
          $srcFile = $srcFileNodeset->nodeset[0];
          // update station number in factory.xml and save changes
          $factoryRoot->set_attribute('station', $station);
          $factoryRoot->dump_file(($srcDirectory . $srcFilename), false, true);
        }
        $scriptFilepath = ($srcDirectory . $srcFile->get_attribute('name'));
      }
      else
      {
        $scriptFilepath = ($srcDirectory . $srcFilename);
      }
      // Check for errors
      if (!checkDiagConfig('/factory/ignoreErrorLog'))
      {
        require_once 'aibridge.class.php';
        $errorLog = new DomDocument(AIBridge::getErrorLog());
        $xpath = xpath_new_context($errorLog->document_element());
        $errorLogRecords = @xpath_eval($xpath, '/errorLogRecord');
        if (count($errorLogRecords->nodeset) > 0)
          define('FACTORY_ERRORS_FOUND', 1);
      }
      if (!defined('FACTORY_ERRORS_FOUND'))
      {
        if (!file_exists($scriptFilepath))
        {
          define('SCRIPT_FILE_NOT_FOUND', 1);
        }
        else
        {
          $scriptFile = new DomDocument(file_get_contents($scriptFilepath));
          $xpath = xpath_new_context($scriptFile->document_element());
          $diagRun = @ xpath_eval($xpath, '/diagRun');
          $tests = @ xpath_eval($xpath, '/diagRun/runTest');
          if (count($tests->nodeset) <= 0)
          {
            define('NO_TESTS_FOUND_IN_SCRIPT', 1);
          }
          else
          {
            $testsArray = array();
            foreach ($tests->nodeset as $test)
            {
              $testArray = array();
              $testArray['device'] = $test->get_attribute('device');
              $testArray['test'] = $test->get_attribute('test');
              $testArray['parameters'] = array();
              if ($test->has_attributes())
              {
                $testAttributes = $test->attributes();
                foreach ($testAttributes as $attribute)
                {
                  $attributeArray = array();
                  $attributeArray['name'] = $attribute->name();
                  $attributeArray['value'] = $attribute->value();
                  $testArray['parameters'][] = $attributeArray;
                }
              }
              $testsArray[] = $testArray;
            }
            if (count($diagRun->nodeset) > 0)
              $caption = $diagRun->nodeset[0]->get_attribute('jobCaption');
            else
              $caption = '';
            require_once 'aibridge.class.php';
            $stdout = AIBridge::startCustomTests($caption, '', '', '', $testsArray, true);
            $xml = new DomDocument($stdout);
            if ($xml)
            {
              $root = $xml->document_element();
              if ($root->tagname() != 'error')
              {
                define('SCRIPT_TESTING', true);
                define('SCRIPT_TESTING_PID', $root->get_attribute('processID'));
              }
              else
              {
                define('SCRIPT_TESTING', false);
                define('SCRIPT_TESTING_ERROR', $root->get_content());
              }
            }
            else
            {
              define('SCRIPT_TESTING', false);
              define('SCRIPT_TESTING_ERROR', ('XMLDOC Error: ' . $stdout));
            }
          }
        }
        define('SCRIPT_TESTING', false);
      }
    }

    if (DEBUG_MODE == true)
    {
      $constants = getUserConstants();
      $debugFile = fopen(FRONTEND_DEBUG_FILE, 'w');
      fwrite($debugFile, (date('Y-m-d H:i:s') . "\nFrontend has started ...\nConstants:\n" . print_r($constants, true) . "\n\n"));
      fclose($debugFile);
    }
  }

  function getUserConstants()
  {
    global $systemConstants;
    $countOfSystemConstants = count($systemConstants);
    $constants = get_defined_constants();
    $userConstants = array_slice($constants, $countOfSystemConstants);
    return $userConstants;
  }

  function isImlSupported()
  {
    $returnVal = false;
    $xml = new DomDocument(file_get_contents(IDRESULT_PATH));
    $xpath = xpath_new_context($xml->document_element());
    $imlLogEntry = @ xpath_eval($xpath, '/diagOutput/category[@name="miscellaneous"]/structure[@name="imlLog"]');
    if (count($imlLogEntry->nodeset) > 0)
    {
      $caption = $imlLogEntry->nodeset[0]->get_attribute('caption');
      if (strstr($caption, 'Not available') === false)
        $returnVal = true;
    }
    return $returnVal;
  }

  function isSurveyOnly()
  {
    $returnVal = false;
    if (IS_FACTORY == false && ADVANCED_MODE == false)
    {
      if (checkDiagConfig('/mode', 'surveyOnly', '1'))
      {
        if (checkDiagConfig('/mode', 'envior', 'online') && IS_ONLINE)
          $returnVal = true;
        else if (checkDiagConfig('/mode', 'envior', 'offline') && IS_PHOENIX)
          $returnVal = true;
      }
    }
    return $returnVal;
  }

  function hasDiagnosableDevices()
  {
    $returnVal = false;
    $xml = new DomDocument(file_get_contents(IDRESULT_PATH));
    $xpath = xpath_new_context($xml->document_element());
    $diagnosableDevices = @ xpath_eval($xpath, '/diagOutput/category/device[@diagnosable="1"]');
    if (count($diagnosableDevices->nodeset))
      $returnVal = true;
    return $returnVal;
  }

  // Check the $_REQUEST global for the given list of parameters, accepts multiple strings
  function requiredParameters()
  {
    // We default to a true response
    $returnVal = true;
    // Get the passed arguments, must be done this way to accomadate variable length argument list
    $arguments = func_get_args();
    foreach ($arguments as $parameter)
    {
      // If any parameter is not found, returnVal becomes false
      if (!isset($_REQUEST[$parameter]))
        $returnVal = false;
    }
    return $returnVal;
  }

  function getComputerName()
  {
    $computerName = '';
    if (file_exists(IDRESULT_PATH))
    {
      $xml = new DomDocument(file_get_contents(IDRESULT_PATH));
      $xpath = xpath_new_context($xml->document_element());
      $hostNameNodes = @ xpath_eval($xpath, '/diagOutput/category[@name="miscellaneous"]/structure[@name="systemSummary"]/property[@name="hostName"]');
      if (count($hostNameNodes->nodeset) > 0)
        $computerName = $hostNameNodes->nodeset[0]->get_attribute('value');
    }
    return $computerName;
  }

  function getBuildVersion()
  {
    $buildVersion = '';
    if (file_exists(IDRESULT_PATH))
    {
      $xml = new DomDocument(file_get_contents(IDRESULT_PATH));
      $xpath = xpath_new_context($xml->document_element());
      $diagOutputNodes = @ xpath_eval($xpath, '/diagOutput');
      if (count($diagOutputNodes->nodeset) > 0)
        $buildVersion = $diagOutputNodes->nodeset[0]->get_attribute('buildVersion');
    }
    return $buildVersion;
  }

  function extractTags()
  {
    $arguments = func_get_args();
    $returnVal = array();
    $xmlString = $arguments[0];
    $xml = new DomDocument($xmlString);
    $xpath = xpath_new_context($xml->document_element());
    for ($i=1; $i<count($arguments); $i++)
    {
      $tagname = $arguments[$i];
      $elements = @ xpath_eval($xpath, '/'.$tagname);
      $tag = $elements->nodeset[0];
      if ($tag)
        $returnVal[$tagname] = $tag->get_content();
      else
        $returnVal[$tagname] = '';
    }
    return $returnVal;
  }

  function echoln($msg)
  {
    echo ($msg . "\n");
  }

  function writeArguments()
  {
    if (file_exists(FRONTEND_DEBUG_FILE))
    {
      $size = filesize(FRONTEND_DEBUG_FILE);
      // if debug file is greater than 1/2 megabyte in size, clear and refresh
      if ($size > 524288)
      {
        $constants = getUserConstants();
        $debugFile = fopen(FRONTEND_DEBUG_FILE, 'w');
        fwrite($debugFile, (date('Y-m-d H:i:s') . "\nDebug File Cleared\nConstants:\n" . print_r($constants, true) . "\n"));
      }
    }
    if (count($_GET) > 0)
      $GET = ("GET:\n" . print_r($_GET, true));
    else
      $GET = "GET: empty\n";
    if (count($_POST) > 0)
      $POST = ("POST:\n" . print_r($_POST, true));
    else
      $POST = "POST: empty\n";
    $debugFile = fopen(FRONTEND_DEBUG_FILE, 'a');
    fwrite($debugFile, (date('Y-m-d H:i:s') . "\n" . $_SERVER['PHP_SELF'] . ": arguments:\n" . $GET . $POST . "\n"));
    fclose($debugFile);
  }
  
  function appendViewHeader(&$output)
  {
    $output[] = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"';
    $output[] = '"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">';
    $output[] = '<html xmlns="http://www.w3.org/1999/xhtml">';
    $output[] = '<head>';
    $output[] = '<title>view</title>';
    $output[] = '<link type="text/css" rel="stylesheet" media="all" href="../css/views.css"></link>';
  }

  function outputViewHeader()
  {
    $output = array();
    appendViewHeader($output);
    echo implode('', $output);
  }
  
  function appendViewFooter(&$output)
  {
    $output[] = '</body>';
    $output[] = '</html>';
  }
  
  function outputViewFooter()
  {
    $output = array();
    appendViewFooter($output);
    echo implode('', $output);
  }

  function buildJsConfig()
  {
    require_once 'rosetta.class.php';
    $CONFIG = array();

    $CONFIG['DEBUG_MODE'] = (DEBUG_MODE === true) ? true : false;
    $CONFIG['ADVANCED_MODE'] = (ADVANCED_MODE === true) ? true : false;
    $CONFIG['SURVEY_ONLY'] = (SURVEY_ONLY === true) ? true : false;
    $CONFIG['IS_WINDOWS'] = (IS_WINDOWS === true) ? true : false;
    $CONFIG['IS_LINUX'] = (IS_LINUX === true) ? true : false;
    $CONFIG['IS_OFFLINE'] = (IS_OFFLINE === true) ? true : false;
    $CONFIG['IS_ONLINE'] = (IS_ONLINE === true) ? true : false;
    $CONFIG['IS_FACTORY'] = (IS_FACTORY === true) ? true : false;
    $CONFIG['IML_SUPPORT'] = (IML_SUPPORT === true) ? true : false;
    $CONFIG['DIAGNOSE_SUPPORT'] = (DIAGNOSE_SUPPORT === true) ? true : false;
    $CONFIG['BASE_URI'] = FRONTEND_DIRECTORY;

    if (defined('OVERRIDE_LANG'))
      $CONFIG['OVERRIDE_LANG'] = OVERRIDE_LANG;

    if (defined('SCRIPT_TESTING') && SCRIPT_TESTING === true)
      $CONFIG['SCRIPT_TESTING_PID'] = SCRIPT_TESTING_PID;

    $CONFIG['FACTORY_ERRORS_FOUND'] = (defined('FACTORY_ERRORS_FOUND') && FACTORY_ERRORS_FOUND === true) ? true : false;
    
    if (isset($_GET['ignoreHelp']))
      $CONFIG['HELP_SRC'] = 'utils/dummy.html';
    else if (IS_WINDOWS === true)
      $CONFIG['HELP_SRC'] = 'help/windows/online/InsightDiags.htm';
    else if (IS_OFFLINE === true)
      $CONFIG['HELP_SRC'] = 'help/linux/offline/InsightDiags.htm';
    else
      $CONFIG['HELP_SRC'] = 'help/linux/online/InsightDiags.htm';

    return Rosetta::arrayToJSON($CONFIG);
  }

  // DEBUG OUTPUT - if we are in debug mode, output all arguments to debug file
  if (DEBUG_MODE == true)
    writeArguments();
?>