<?php
  header('Content-type: text/html; charset=utf-8', true);
  header('Cache-control: no-cache', true);
  header('Pragma: no-cache');
    
  include_once '../includes/globals.php';
  include_once '../includes/aibridge.class.php';
  
  $leftFilepath = $_GET['leftFilepath'];
  $leftFilename = $_GET['leftFilename'];
  $rightFilepath = $_GET['rightFilepath'];
  $rightFilename = $_GET['rightFilename'];
  $selectedCategory = (isset($_GET['category'])) ? $_GET['category'] : 'all';
  $changesOnly = (isset($_GET['changesOnly']) && $_GET['changesOnly'] == 'yes') ? true : false;
  $advanced = (isset($_GET['advanced']) && $_GET['advanced'] == 'yes') ? true : false;
  $tableClass = array('config');
  $tableClass[] = ($advanced == true) ? 'advanced' : 'summary';
  $tableClass[] = ($changesOnly == true) ? 'changesOnly' : '';

  // Check for files
  if (!file_exists(DIAGS_DIRECTORY . $leftFilepath) || !file_exists(DIAGS_DIRECTORY . $rightFilepath))
  {
    $filesMissing = true;
  }
  else
  {
    // Call AI
    $stdout = AIBridge::compareSurveys($leftFilepath, $rightFilepath);
   
    // Try to make stdout into a DomDocument
    $xml = new DomDocument($stdout);
    if ($xml)
    {
      $document = $xml->document_element();
      if ($document->tagname() == 'error') // If AI returned an error:
      {
        exit($document->get_content());
      }
      $filesMatch = $document->get_attribute('filesMatch');
    }
    else
    {
      exit('XML Error - ' . $stdout);
    }
  }
  
  outputViewHeader();
  echo '<script type="text/javascript" src="../js/lib.js"></script>';
  echo '<script type="text/javascript" src="js/compareConfigs.js"></script>';
  echo '</head>';
  if ($filesMissing)
    $arg = 'true';
  else
    $arg = 'false';
  echo ('<body onload="init(' . $arg . ');">');
  echo ('<table class="' . implode(' ', $tableClass) . '" id="compareConfigs">');
  echo '<tbody>';

  if ($filesMissing)
  {
    echo '<tr class="message">';
    echo ('<td>' . translate('MSG_SOME_COMPARE_FILES_MISSING') . '</td>');
    echo '</tr>'; 
  }
  else if ($filesMatch == '1')
  {
    echo '<tr class="message">';
    echo ('<td>' . translate('MSG_NO_CHANGES_FOUND') . '</td>');
    echo '</tr>'; 
  }
  else
  {
    // Get results of the compare
    AIBridge::changeComparisonPermissions();
    $leftCompare  = file_get_contents(DIAGS_DIRECTORY . 'surveycompare.left.xml');
    $rightCompare = file_get_contents(DIAGS_DIRECTORY . 'surveycompare.right.xml');
     
    // Change results into DomDocuments
    $leftXML  = new DomDocument($leftCompare);
    $rightXML = new DomDocument($rightCompare);
      
    // Setup XPath Contexts
    $leftXpath  = xpath_new_context($leftXML->document_element());
    $rightXpath = xpath_new_context($rightXML->document_element());
      
    // Get Categories (right file used as template)
    if ($selectedCategory == 'all')
      $rightCategoryNodes = @ xpath_eval($rightXpath, '/category');
    else
      $rightCategoryNodes = @ xpath_eval($rightXpath, ('/category[@name="' . $selectedCategory . '"]'));

    foreach ($rightCategoryNodes->nodeset as $rightCategory)
    {
      $leftCategoryNodes = @ xpath_eval($leftXpath, ('/category[@name="' . $rightCategory->get_attribute('name') . '"]'));
      $leftCategory = $leftCategoryNodes->nodeset[0];
      
      $rightChildren = $rightCategory->child_nodes();
      $leftChildren = $leftCategory->child_nodes();
      
      $numberOfChildren = count($rightChildren);

      if ($selectedCategory == 'all')
      {
        $rightCategoryXpath = xpath_new_context($rightCategory);
        $nestedChanges = @ xpath_eval($rightCategoryXpath, '//*[@compareResult]');
        $containsChanges = (count($nestedChanges->nodeset) > 0) ? true : false;
        displayCategoryHeader($rightCategory->get_attribute('caption'), $leftFilename, $rightFilename, $containsChanges);
      }
      
      for ($i=0; $i<$numberOfChildren; $i++)
      {
        $rightChild = $rightChildren[$i];
        if (validChild($rightChild))
        {
          $leftChild = $leftChildren[$i];
          $sublevel = ($selectedCategory == 'all') ? 1 : 0;
          displayEntry($rightChild, $leftChild, $sublevel, $output, $rightFilename, $leftFilename);
        }
      }
    }
  }
  
  echo '</tbody>';
  echo '</table>';
  outputViewFooter();
    
  // UTILITY FUNCTIONS

  function validChild($child)
  {
    $valid = false;
    if ($child->node_type() == XML_ELEMENT_NODE)
    {
      $validTagnames = array('device', 'structure', 'property');
      $tagname = $child->tagname();
      $testOnly = $child->get_attribute('testOnly');
      if ($testOnly != '1' && in_array($tagname, $validTagnames))
        $valid = true;
    }
    return $valid;
  }

  function displayCategoryHeader($categoryName, $leftFilename, $rightFilename, $containsChanges)
  {
    $leftFilename = ($leftFilename == 'original') ? translate('MSG_ORIGINAL_CONFIGURATION') : $leftFilename;
    $rightFilename = ($rightFilename == 'current') ? translate('MSG_CURRENT_CONFIGURATION') : $rightFilename;
    $changeClass = ($containsChanges === true) ? '' : 'unchanged';
    echo ('<tr class="header ' . $changeClass . '">');
    echo '<td class="icon"></td>';
    echo ('<th>' . $categoryName . '</th>');
    echo ('<th class="value">' . $leftFilename . '</th>');
    echo ('<th class="value">' . $rightFilename . '</th>');
    echo '</tr>';
  }
  
  function displayEntry($rightEntry, $leftEntry, $sublevel, &$output, $rightFilename, $leftFilename)
  {
    if ($sublevel == 0)
    {
      $xpath = xpath_new_context($rightEntry);
      $nestedChanges = @ xpath_eval($xpath, '//*[@compareResult]');
      $containsChanges = (count($nestedChanges->nodeset) > 0) ? true : false;
      displayHeaderEntry($rightEntry, $output, $rightFilename, $leftFilename, $containsChanges);
    }
    else
    {
      if ($rightEntry->has_attribute('compareResult'))
        $compareResult = $rightEntry->get_attribute('compareResult');
      else
        $compareResult = 'unchanged';
      switch ($compareResult)
      {
      case 'added':
        displayAddedEntry($rightEntry, $sublevel, $output);
        break;
      case 'removed':
        displayRemovedEntry($leftEntry, $sublevel, $output);
        break;
      default:
        displayStandardEntry($rightEntry, $leftEntry, $sublevel, $output, $compareResult);
      }
    }
    
    $rightChildren = $rightEntry->child_nodes();
    $leftChildren = $leftEntry->child_nodes();
    
    $numberOfChildren = count($rightChildren);
    for ($i=0; $i<$numberOfChildren; $i++)
    {
      $rightChild = $rightChildren[$i];
      if (validChild($rightChild))
      {
        $leftChild = $leftChildren[$i];
        $sublevel++;
        displayEntry($rightChild, $leftChild, $sublevel, $output, $rightFilename, $leftFilename);
        $sublevel--;
      }
    }
  }
  
  function displayHeaderEntry($data, &$output, $rightName, $leftName, $containsChanges=false)
  {
    $leftName = ($leftName == 'original') ? translate('MSG_ORIGINAL_CONFIGURATION') : $leftName;
    $rightName = ($rightName == 'current') ? translate('MSG_CURRENT_CONFIGURATION') : $rightName;
    $caption = $data->get_attribute('caption');
    $class = array('header');
    if ($containsChanges == false)
      $class[] = 'unchanged';
    echo ('<tr class="' . implode(' ', $class) . '">');
    echo '<td class="icon"></td>';
    echo ('<th>' . $caption . '</th>');
    echo ('<th class="value">' . $leftName . '</th>');
    echo ('<th class="value">' . $rightName . '</th>');
    echo '</tr>';
  }
  
  function displayAddedEntry($data, $sublevel, &$output)
  {
    $caption = $data->get_attribute('caption');
    $description = $data->get_attribute('description');
    $value = $data->get_attribute('value');
    $hexData = $data->get_attribute('hexData');
    if ($value != '' && $value != $caption)
      $displayValue = $value;
    else if ($description != $caption)
      $displayValue = $description;
    else
      $displayValue = '';
    $class = array('changed');
    if ($hexData == '1')
      $class[] = 'hexdata';
    else if ($sublevel == 1)
      $class[] = 'data';
    else
      $class[] = 'subdata';
    if (in_array('subdata', $class))
      $style = ('style="padding-left: ' . (15 * ($sublevel - 1)) . 'px"');
    else
      $style = '';
    echo ('<tr class="' . implode(' ', $class) . '">');
    echo '<td class="icon"><img src="../images/change_marker.gif"></td>';
    echo ('<th ' . $style . '>' . $caption . '</th>');
    echo ('<td class="value compareResult">(' . translate('MSG_ADDED') . ')</td>');
    echo ('<td class="value">' . $displayValue . '</td>');
    echo '</tr>';
  }
  
  function displayRemovedEntry($data, $sublevel, &$output)
  {
    $caption = $data->get_attribute('caption');
    $description = $data->get_attribute('description');
    $value = $data->get_attribute('value');
    $hexData = $data->get_attribute('hexData');
    if ($value != '' && $value != $caption)
      $displayValue = $value;
    else if ($description != $caption)
      $displayValue = $description;
    else
      $displayValue = '';
    $class = array('changed');
    if ($hexData == '1')
      $class[] = 'hexdata';
    else if ($sublevel == 1)
      $class[] = 'data';
    else
      $class[] = 'subdata';
    if (in_array('subdata', $class))
      $style = ('style="padding-left: ' . (15 * ($sublevel - 1)) . 'px"');
    else
      $style = '';
    echo ('<tr class="' . implode(' ', $class) . '">');
    echo '<td class="icon"><img src="../images/change_marker.gif"></td>';
    echo ('<th ' . $style . '>' . $caption . '</th>');
    echo ('<td class="value">' . $displayValue . '</td>');
    echo ('<td class="value compareResult">(' . translate('MSG_REMOVED') . ')</td>');
    echo '</tr>';
  }
  
  function displayStandardEntry($rightData, $leftData, $sublevel, &$output, $compareResult)
  {
    $caption = $rightData->get_attribute('caption');
    $hexData = $rightData->get_attribute('hexData');
    $rightDescription = $rightData->get_attribute('description');
    $rightValue = $rightData->get_attribute('value');
    
    $leftDescription = $leftData->get_attribute('description');
    $leftValue = $leftData->get_attribute('value');
    
    if ($rightValue != '' && $rightValue != $caption)
      $rightDisplayValue = $rightValue;
    else if ($rightDescription != $caption)
      $rightDisplayValue = $rightDescription;
    else
      $rightDisplayValue = '';
    
    if ($leftValue != '' && $leftValue != $caption)
      $leftDisplayValue = $leftValue;
    else if ($leftDescription != $caption)
      $leftDisplayValue = $leftDescription;
    else
      $leftDisplayValue = '';
    
    $class = array($compareResult);
    if ($hexData == '1')
      $class[] = 'hexdata';
    else if ($sublevel == 1)
      $class[] = 'data';
    else
      $class[] = 'subdata';
      
    if ($compareResult == 'unchanged')
    {
      if ($rightData->get_attribute('techDetail') == '1')
        $class[] = 'advanced';
    }
    
    if (in_array('subdata', $class))
      $style = ('style="padding-left: ' . (15 * ($sublevel - 1)) . 'px"');
    else
      $style = '';
      
    if ($compareResult != 'unchanged')
      $icon = '<img src="../images/change_marker.gif">';
    else
      $icon = '';
      
    echo ('<tr class="' . implode(' ', $class) . '">');
    echo ('<td class="icon">' . $icon . '</td>');
    echo ('<th ' . $style . '>' . $caption . '</th>');
    echo ('<td class="value">' . $leftDisplayValue . '</td>');
    echo ('<td class="value">' . $rightDisplayValue . '</td>');
    echo '</tr>';
  }