<?php
  header('Content-type: text/html; charset=utf-8', true);
  header('Cache-control: no-cache', true);
  header('Pragma: no-cache');

  include_once '../includes/globals.php';
  
  ini_set('error_reporting', E_ALL);
  ini_set('display_errors', 1);
  
  $selectedCategory = (isset($_GET['category'])) ? $_GET['category'] : 'all';
  $tableClass = (isset($_GET['advanced']) && $_GET['advanced'] == 'yes') ? 'advanced' : 'summary';
  
  outputViewHeader();
  echo '<script type="text/javascript" src="../js/lib.js"></script>';
  echo '<script type="text/javascript" src="js/currentConfig.js"></script>';
  echo '</head>';
  echo '<body onload="init();">';
  echo ('<table class="config ' . $tableClass . '" id="currentConfig">');
  echo '<tbody>';
  
  // Check for ID result
  $filepath = (IDRESULT_PATH);
  if (!file_exists($filepath))
  {
    echo '<tr class="message">';
    echo '<td>IDRESULT MISSING</td>';
    echo '</tr>';
  }
  else
  {
    // Make ID result into a DomDocument
    $xml = new DomDocument(file_get_contents($filepath));
    $xpath = xpath_new_context($xml->document_element());
    $categories = @ xpath_eval($xpath, '/diagOutput/category');
    
    foreach($categories->nodeset as $category)
    {
      $xpath = xpath_new_context($category);
      $categoryName = $category->get_attribute('name');
      if ($selectedCategory == 'all' || $categoryName == $selectedCategory)
      {
        $categoryCaption = $category->get_attribute('caption');
        $categoryChildren = @ xpath_eval($xpath, '/child::*');
        if ($selectedCategory == 'all')
          displayCategoryHeader($categoryCaption);
        foreach($categoryChildren->nodeset as $categoryChild)
        {
          $tagname = $categoryChild->tagname;
          $testOnly = $categoryChild->get_attribute('testOnly');
          // Make sure we want to display this entry
          if ($testOnly != '1' && 
              ($tagname == 'device' || 
               $tagname == 'structure' || 
               $tagname == 'property'))
          {
            $sublevel = ($selectedCategory == 'all') ? 1 : 0;
            displayEntry($categoryChild, $sublevel);
          }
        }
      }
    }
  }
  
  echo '</tbody>';
  echo '</table>';
  outputViewFooter();

  function displayCategoryHeader($name)
  {
    echo '<tr class="header">';
    echo ('<th colspan="2">' . $name . '</th>');
    echo '</tr>';
  }

  function displayEntry($entry, $sublevel, $advanced=false)
  {
    $xpath = xpath_new_context($entry);
    
    $caption = $entry->get_attribute('caption');
    $value = $entry->get_attribute('value');
    $description = $entry->get_attribute('description');
    $actualValue = ($value != '') ? $value : (($description != '') ? $description : '');
    $isAdvanced = ($advanced || $entry->get_attribute('techDetail') == '1') ? true : false;
    $isHex = ($entry->get_attribute('hexData') == '1') ? true : false;
  
    if ($sublevel == 0)
    {
      echo '<tr class="header">';
      if ($actualValue == '' || $actualValue == $caption)
        echo ('<th colspan="2">' . $caption . '</th>');
      else
        echo ('<th colspan="2">' . $caption . ' - ' . $actualValue . '</th>');
      echo '</tr>';
    }
    else if ($isHex)
    {
      $class = ($isAdvanced) ? 'hexdata advanced' : 'hexdata';
      echo ('<tr class="' . $class . '">');
      echo ('<th colspan="2">' . $caption . ' : ' . $actualValue . '</th>');
      echo '</tr>';
    }
    else if ($sublevel == 1)
    {
      $class = ($isAdvanced) ? 'data advanced' : 'data';
      echo ('<tr class="' . $class . '">');
      if ($actualValue == '' || $actualValue == $caption)
      {
	      echo ('<th colspan="2">' . $caption . '</th>');
      }
	    else
	    {
	      echo ('<th>' . $caption . '</th>');
	      echo ('<td>' . $actualValue . '</td>');
	    }
	    echo '</tr>';
	  }
	  else
	  {
	    $padding = (15 * ($sublevel - 1));
      $class = ($isAdvanced) ? 'subdata advanced' : 'subdata';
	    echo ('<tr class="' . $class . '">');
	    if ($actualValue == '' || $actualValue == $caption)
	    {
	      echo ('<th colspan="2" style="padding-left: ' . $padding . 'px">' . $caption . '</th>');
	    }
	    else
	    {
        echo ('<th style="padding-left: ' . $padding . 'px">' . $caption . '</th>');
	      echo ('<td>' . $actualValue . '</td>');
	    }
	    echo '</tr>';
	  }
    if ($entry->has_child_nodes())
    {
      $children = @ xpath_eval($xpath, '/child::node()');
      foreach ($children->nodeset as $child)
      {
        // Make sure child is an XML_ELEMENT_NODE
        if ($child->node_type() == XML_ELEMENT_NODE)
        {
          $tagname = $child->tagname;
          $testOnly = $child->get_attribute('testOnly');
          // Make sure we want to display this entry
          if ($testOnly != '1' && 
              ($tagname == 'device' || 
               $tagname == 'structure' || 
               $tagname == 'property'))
          {
            // If this element is advanced, all of its children need to be marked advanced as well so they are hidden or visible along with their parent
            $nestedAdvanced = ($advanced || $isAdvanced) ? true : false;
            $sublevel++;
            displayEntry($child, $sublevel, $nestedAdvanced);
            $sublevel--; // put sublevel back to original number on return
          }
        }
      }
    }
  }
  //*/
?>
